#
# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# NVIDIA CORPORATION and its licensors retain all intellectual property
# and proprietary rights in and to this software, related documentation
# and any modifications thereto.  Any use, reproduction, disclosure or
# distribution of this software and related documentation without an express
# license agreement from NVIDIA CORPORATION is strictly prohibited.
#

find_package(PackageHandleStandardArgs)

if (WIN32 AND NOT DEFINED FXC_EXECUTABLE)

    # locate Win 10 kits
    get_filename_component(kit10_dir "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows Kits\\Installed Roots;KitsRoot10]" REALPATH)
    file(GLOB W10SDK_VERSIONS RELATIVE ${kit10_dir}/Include ${kit10_dir}/Include/10.*)          # enumerate pre-release and not yet known release versions
    list(APPEND W10SDK_VERSIONS "10.0.15063.0" "10.0.16299.0" "10.0.17134.0")   # enumerate well known release versions
    list(REMOVE_DUPLICATES W10SDK_VERSIONS)
    list(SORT W10SDK_VERSIONS)    # sort from low to high
    list(REVERSE W10SDK_VERSIONS) # reverse to start from high

    foreach(W10SDK_VER ${W10SDK_VERSIONS})

        set(WINSDK_PATHS "${kit10_dir}/bin/${W10SDK_VER}/x64" "C:/Program Files (x86)/Windows Kits/10/bin/${W10SDK_VER}/x64" "C:/Program Files/Windows Kits/10/bin/${W10SDK_VER}/x64")

        find_program(FXC_EXECUTABLE fxc PATHS ${WINSDK_PATHS})

        if (EXISTS ${FXC_EXECUTABLE})

            set(FXC_VERSION "${W10SDK_VER}")
            break()
        endif()

    endforeach()
endif()

include(FindPackageHandleStandardArgs)

find_package_handle_standard_args(fxc
    REQUIRED_VARS
        FXC_EXECUTABLE
    VERSION_VAR
        FXC_VERSION
)

