/*
 * Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
 *
 * NVIDIA CORPORATION and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA CORPORATION is strictly prohibited.
 */
 
#pragma once

#include <applib/DeviceManager.h>
#include <applib/imgui_nvrhi.h>
#include <memory>

// base class to build IRenderPass-based UIs using ImGui through NVRHI
class ImGui_Renderer : public IRenderPass
{
protected:
    std::unique_ptr<ImGui_NVRHI> imgui_nvrhi;

    // buffer mouse click and keypress events to make sure we don't lose events which last less than a full frame
    std::array<bool, 3> mouseDown = { false };
    std::array<bool, GLFW_KEY_LAST + 1> keyDown = { false };

public:
    ImGui_Renderer(DeviceManager *devManager);
    ~ImGui_Renderer();
    bool init();

    virtual bool KeyboardUpdate(int key, int scancode, int action, int mods) override;
    virtual bool MousePosUpdate(double xpos, double ypos) override;
    virtual bool MouseScrollUpdate(double xoffset, double yoffset) override;
    virtual bool MouseButtonUpdate(int button, int action, int mods) override;
    virtual void Animate(float elapsedTimeSeconds) override;
    virtual void Render(nvrhi::ITexture* renderTarget, uint32_t swapChainIndex) override;
    virtual void BackBufferResizing() override;

protected:
    // creates the UI in ImGui, updates internal UI state
    virtual void buildUI(void) = 0;
};
